{**********************************************}
{                                              }
{             FastReport v5.0                  }
{        NativeDB enduser components           }
{                                              }
{         Copyright (c) 1998-2014              }
{           by Fast Reports Inc.               }
{                                              }
{ Copyright (c) 2000-2014 by Stalker SoftWare  }
{                                              }
{**********************************************}

unit frxASAComponents;

interface

{$I frx.inc}

uses
  Windows, Classes, SysUtils, frxClass, frxCustomDB, DB, Graphics,
  NdbBasDS, NdbBase, NdbAsa, NdbAsaDS, frxDsgnIntf, frxRes, frxUtils
{$IFDEF Delphi6}
, Variants
{$ENDIF}
{$IFDEF QBUILDER}
, ndbApp, fqbClass
{$ENDIF};

type
{$IFDEF DELPHI16}
[ComponentPlatformsAttribute(pidWin32 or pidWin64)]
{$ENDIF}
  TfrxASAComponents = class(TfrxDBComponents)
  private
  public
    function GetDescription: String; override;
  end; { TfrxASAComponents }


  { TfrxAsaSession }

  TfrxAsaSession = class(TfrxCustomDatabase)
  private
    FASASession: TASASession;
    function GetServerType: TASAServerType;
    procedure SetServerType(const Value: TAsaServerType);
    function GetKeepConnection: Boolean;
    procedure SetKeepConnection(const Value: Boolean);
    function GetTransIsolation: TAsaTransIsolation;
    procedure SetTransIsolation(const Value: TAsaTransIsolation);
    function GetAutoCommit: Boolean;
    function GetClientParams: String;
    function GetCommitOnDisconnect: Boolean;
    function GetLibraryFile: String;
    function GetLibraryPath: String;
    function GetLoginDataBase: String;
    function GetLoginEngineName: String;
    function GetLoginPassword: String;
    function GetLoginUser: String;
    function GetOEMConvert: Boolean;
    function GetServerParams: String;
    function GetTrimBlanks: Boolean;
    function GetWantExceptions: Boolean;
    function GetWantRowCounts: Boolean;
    procedure SetAutoCommit(const Value: Boolean);
    procedure SetClientParams(const Value: String);
    procedure SetCommitOnDisconnect(const Value: Boolean);
    procedure SetLibraryFile(const Value: String);
    procedure SetLibraryPath(const Value: String);
    procedure SetLoginDataBase(const Value: String);
    procedure SetLoginEngineName(const Value: String);
    procedure SetLoginPassword(const Value: String);
    procedure SetLoginUser(const Value: String);
    procedure SetOEMConvert(const Value: Boolean);
    procedure SetServerParams(const Value: String);
    procedure SetTrimBlanks(const Value: Boolean);
    procedure SetWantExceptions(const Value: Boolean);
    procedure SetWantRowCounts(const Value: Boolean);

  protected
    procedure SetConnected(Value: Boolean); override;
    function GetConnected: Boolean; override;
    procedure SetName(const AName: TComponentName); override;

  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    property ASASession: TASASession read FASASession;

  published
    property KeepConnection :Boolean read GetKeepConnection write SetKeepConnection default True;
    property ServerType :TAsaServerType read GetServerType write SetServerType default stClient;
    property TransIsolation :TAsaTransIsolation read GetTransIsolation write SetTransIsolation default atUnchanged;
    property AutoCommit :Boolean read GetAutoCommit write SetAutoCommit default False;
    property CommitOnDisconnect :Boolean read GetCommitOnDisconnect write SetCommitOnDisconnect default True;
    property ClientParams :String read GetClientParams write SetClientParams;
    property LibraryFile :String read GetLibraryFile write SetLibraryFile;
    property LibraryPath :String read GetLibraryPath write SetLibraryPath;
    property LoginDataBase :String read GetLoginDataBase write SetLoginDataBase;
    property LoginEngineName :String read GetLoginEngineName write SetLoginEngineName;
    property LoginUser :String read GetLoginUser write SetLoginUser;
    property LoginPassword :String read GetLoginPassword write SetLoginPassword;
    property OEMConvert :Boolean read GetOEMConvert write SetOEMConvert default False;
    property ServerParams :String read GetServerParams write SetServerParams;
    property WantExceptions :Boolean read GetWantExceptions write SetWantExceptions default False;
    property WantRowCounts :Boolean read GetWantRowCounts write SetWantRowCounts default True;
    property TrimBlanks  :Boolean read GetTrimBlanks write SetTrimBlanks default False;
    property Connected: Boolean read GetConnected write SetConnected stored False default False;

  end; { TfrxAsaSession }

  TfrxAsaDataSet = class(TfrxCustomQuery)
  private
    FQuery       :TAsaDataSet;
    FSessionName :String;
    function GetParamCheck: Boolean;
    procedure SetParamCheck(const Value: Boolean);
    function GetSession :TNativeSession;
    procedure SetSession(Value: TNativeSession);
    function GetAutoCalcFields: Boolean;
    procedure SetAutoCalcFields(const Value: Boolean);
    function GetReadOnly: Boolean;
    procedure SetReadOnly(const Value: Boolean);
    function GetCursorType: TAsaCursorType;
    function GetSyncInserts: Boolean;
    procedure SetCursorType(const Value: TAsaCursorType);
    procedure SetSyncInserts(const Value: Boolean);
    function GetStillInProc: Boolean;
    procedure SetSessionName(const Value: String);

  protected
    procedure SetName(const AName: TComponentName); override;
    procedure SetSQL(Value: TStrings); override;
    function GetSQL: TStrings; override;
    procedure SetMaster(const Value: TDataSource); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure UpdateParams; override;

    function LocateNext(const KeyFields: string; const KeyValues: Variant;Options: TLocateOptions): Boolean;
    procedure ExecSQL;
    procedure Resume;

    property Query: TAsaDataSet read FQuery;
    property StillInProc: Boolean read GetStillInProc;

{$IFDEF QBUILDER}
    function QBEngine: TfqbEngine; override;
{$ENDIF}

  published
    property ParamCheck :Boolean read GetParamCheck write SetParamCheck default True;
    property Session :TNativeSession read GetSession write SetSession;
    property SessionName: String read FSessionName write SetSessionName;
    property AutoCalcFields :Boolean read GetAutoCalcFields write SetAutoCalcFields default True;
    property ReadOnly :Boolean read GetReadOnly write SetReadOnly default False;
    property CursorType :TAsaCursorType read GetCursorType write SetCursorType default actDefault;
    property SyncInserts :Boolean read GetSyncInserts write SetSyncInserts default False;
    property Active stored False default False;
  end; { TfrxAsaDataSet }

{$IFDEF QBUILDER}
  TfrxEngineASA = class(TfqbEngine)
  private
    FQuery :TAsaDataSet;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    
    procedure ReadTableList(ATableList :TStrings); override;
    procedure ReadFieldList(const ATableName :String; var AFieldList :TfqbFieldList); override;
    function ResultDataSet: TDataSet; override;
    procedure SetSQL(const Value :String); override;

    property Query :TAsaDataSet read FQuery;

  end; { TfqbEngine }
{$ENDIF}

implementation

{$R frxASA.RES}

uses
  frxASARTTI
{$IFNDEF NO_EDITORS}
 ,frxASAEditor
{$ENDIF}
;
procedure frxParamsToTParams(Query: TfrxAsaDataSet; Params: TNativeParams); forward;

{ TfrxASAComponents }

function TfrxASAComponents.GetDescription: String;
begin
 Result := 'NativeDB';
end; { GetDescription }

{ TfrxAsaSession }

constructor TfrxAsaSession.Create(AOwner: TComponent);
begin
 inherited;
 FASASession := TASASession.Create(nil);
 FASASession.Name := Self.Name;
 Component := FASASession;
 FASASession.AutoCommit := False;
 FASASession.LibraryFile := 'dblib8.dll';
end; { Create }

class function TfrxAsaSession.GetDescription: String;
begin
 Result := 'TfrxAsaSession';
end; { GetDescription }

procedure TfrxAsaSession.SetName(const AName: TComponentName);
begin
 inherited;
 FASASession.Name := AName;
end; { SetName }

procedure TfrxAsaSession.SetConnected(Value: Boolean);
begin
 BeforeConnect(Value); // for FR 3.20.1
 FASASession.Connected := Value;
end; { SetConnected }

function TfrxAsaSession.GetConnected: Boolean;
begin
 Result := FASASession.Connected;
end; { GetConnected }

function TfrxAsaSession.GetServerType: TAsaServerType;
begin
 Result := FASASession.ServerType;
end; { GetServerType }

procedure TfrxAsaSession.SetServerType(const Value: TAsaServerType);
begin
 FASASession.ServerType := Value;
end; { SetServerType }

function TfrxAsaSession.GetKeepConnection: Boolean;
begin
 Result := FASASession.KeepConnection;
end; { GetKeepConnection }

procedure TfrxAsaSession.SetKeepConnection(const Value: Boolean);
begin
 FASASession.KeepConnection := Value;
end; { SetKeepConnection }

function TfrxAsaSession.GetTransIsolation: TAsaTransIsolation;
begin
 Result := FASASession.TransIsolation;
end; { GetTransIsolation }

procedure TfrxAsaSession.SetTransIsolation(const Value: TAsaTransIsolation);
begin
 FASASession.TransIsolation := Value;
end; { SetTransIsolation }

function TfrxAsaSession.GetAutoCommit: Boolean;
begin
 Result := FASASession.AutoCommit;
end; { GetAutoCommit }

procedure TfrxAsaSession.SetAutoCommit(const Value: Boolean);
begin
 FASASession.AutoCommit := Value;
end; { SetAutoCommit }

function TfrxAsaSession.GetClientParams: String;
begin
 Result := FASASession.ClientParams;
end; { GetClientParams }

procedure TfrxAsaSession.SetClientParams(const Value: String);
begin
 FASASession.ClientParams := Value;
end; { SetClientParams }

function TfrxAsaSession.GetCommitOnDisconnect: Boolean;
begin
 Result := FASASession.CommitOnDisconnect;
end; { GetCommitOnDisconnect }

procedure TfrxAsaSession.SetCommitOnDisconnect(const Value: Boolean);
begin
 FASASession.CommitOnDisconnect := Value;
end; { SetCommitOnDisconnect }

function TfrxAsaSession.GetLibraryFile: String;
begin
 Result := FASASession.LibraryFile;
end; { GetLibraryFile }

procedure TfrxAsaSession.SetLibraryFile(const Value: String);
begin
 FASASession.LibraryFile := Value;
end; { SetLibraryFile }

function TfrxAsaSession.GetLibraryPath: String;
begin
 Result := FASASession.LibraryPath;
end; { GetLibraryPath }

procedure TfrxAsaSession.SetLibraryPath(const Value: String);
begin
 FASASession.LibraryPath := Value;
end; { SetLibraryPath }

function TfrxAsaSession.GetLoginDataBase: String;
begin
 Result := FASASession.LoginDataBase;
end; { GetLoginDataBase }

procedure TfrxAsaSession.SetLoginDataBase(const Value: String);
begin
 FASASession.LoginDataBase := Value;
end; { SetLoginDataBase }

function TfrxAsaSession.GetLoginEngineName: String;
begin
 Result := FASASession.LoginEngineName;
end; { GetLoginEngineName }

procedure TfrxAsaSession.SetLoginEngineName(const Value: String);
begin
 FASASession.LoginEngineName := Value;
end; { SetLoginEngineName }

function TfrxAsaSession.GetLoginPassword: String;
begin
 Result := FASASession.LoginPassword;
end; { GetLoginPassword }

procedure TfrxAsaSession.SetLoginPassword(const Value: String);
begin
 FASASession.LoginPassword := Value;
end; { SetLoginPassword }

function TfrxAsaSession.GetLoginUser: String;
begin
 Result := FASASession.LoginUser;
end; { GetLoginUser }

procedure TfrxAsaSession.SetLoginUser(const Value: String);
begin
 FASASession.LoginUser := Value;
end; { SetLoginUser }

function TfrxAsaSession.GetOEMConvert: Boolean;
begin
 Result := FASASession.OEMConvert;
end; { GetOEMConvert }

procedure TfrxAsaSession.SetOEMConvert(const Value: Boolean);
begin
 FASASession.OEMConvert := Value;
end; { SetOEMConvert }

function TfrxAsaSession.GetServerParams: String;
begin
 Result := FASASession.ServerParams;
end; { GetServerParams }

procedure TfrxAsaSession.SetServerParams(const Value: String);
begin
 FASASession.ServerParams := Value;
end; { SetServerParams }

function TfrxAsaSession.GetTrimBlanks: Boolean;
begin
 Result := FASASession.TrimBlanks;
end; { GetTrimBlanks }

procedure TfrxAsaSession.SetTrimBlanks(const Value: Boolean);
begin
 FASASession.TrimBlanks := Value;
end; { SetTrimBlanks }

function TfrxAsaSession.GetWantExceptions: Boolean;
begin
 Result := FASASession.WantExceptions;
end; { GetWantExceptions }

procedure TfrxAsaSession.SetWantExceptions(const Value: Boolean);
begin
 FASASession.WantExceptions := Value;
end; { SetWantExceptions }

function TfrxAsaSession.GetWantRowCounts: Boolean;
begin
 Result := FASASession.WantRowCounts;
end; { GetWantRowCounts }

procedure TfrxAsaSession.SetWantRowCounts(const Value: Boolean);
begin
 FASASession.WantRowCounts := Value;
end; { SetWantRowCounts }

{ TfrxAsaDataSet }

constructor TfrxAsaDataSet.Create(AOwner: TComponent);
begin
 FQuery := TAsaDataSet.Create(nil);
 Dataset := FQuery;
 FQuery.Name := Self.Name;
 inherited;
end; { Create }

destructor TfrxAsaDataSet.Destroy;
begin
 inherited;
end; { Destroy }

procedure TfrxAsaDataSet.SetName(const AName: TComponentName);
begin
 inherited;
 FQuery.Name := AName;
end; { SetName }

procedure TfrxAsaDataSet.SetMaster(const Value: TDataSource);
begin
 FQuery.DataSource := Value;
end; { SetMaster }

class function TfrxAsaDataSet.GetDescription: String;
begin
 Result := 'TfrxAsaDataSet';
end; { GetDescription }

function TfrxAsaDataSet.GetSQL: TStrings;
begin
 Result := FQuery.SQL;
end; { GetSQL }

procedure TfrxAsaDataSet.SetSQL(Value: TStrings);
begin
 FQuery.SQL.Assign(Value);
end; { GetSQL }

procedure TfrxAsaDataSet.UpdateParams;
begin
 frxParamsToTParams(Self, FQuery.Params);
end; { UpdateParams }

procedure TfrxAsaDataSet.SetSession(Value: TNativeSession);
begin

 if (Value is TAsaSession) or (Value = nil) then
   FQuery.Session := Value
 else
 if TComponent(Value) is TfrxAsaSession then
   FQuery.Session := TfrxAsaSession(Value).AsaSession;

  if FQuery.Session = nil then begin
    if (Report = nil) or (not Report.IsLoading) then
      FSessionName := ''
 end else begin
   if FQuery.Session.Owner = nil then
     FSessionName := FQuery.Session.Name
   else
     FSessionName := FQuery.Session.Owner.Name+'.'+FQuery.Session.Name
 end; { if }

end; { SetSession }

function TfrxAsaDataSet.GetSession :TNativeSession;
begin
 Result := FQuery.Session;
end; { GetSession }

procedure TfrxAsaDataSet.SetSessionName(const Value: String);
var
  oComponent :TComponent;

begin

 FSessionName := Value;
 oComponent := frxFindComponent(Report, FSessionName);

 if (oComponent is TAsaSession) or (oComponent = nil) then
   FQuery.Session := TNativeSession(oComponent)
 else
 if oComponent is TfrxAsaSession then
   FQuery.Session := TfrxAsaSession(oComponent).AsaSession;

end; { SetSessionName }

function TfrxAsaDataSet.GetParamCheck: Boolean;
begin
 Result := FQuery.ParamCheck;
end; { GetParamCheck }

procedure TfrxAsaDataSet.SetParamCheck(const Value: Boolean);
begin
 FQuery.ParamCheck := Value;
end; { SetParamCheck }

function TfrxAsaDataSet.GetReadOnly: Boolean;
begin
 Result := FQuery.ReadOnly;
end; { GetReadOnly }

procedure TfrxAsaDataSet.SetReadOnly(const Value: Boolean);
begin
 FQuery.ReadOnly := Value;
end; { SetReadOnly }

function TfrxAsaDataSet.GetAutoCalcFields: Boolean;
begin
 Result := FQuery.AutoCalcFields;
end; { GetAutoCalcFields }

procedure TfrxAsaDataSet.SetAutoCalcFields(const Value: Boolean);
begin
 FQuery.AutoCalcFields := Value;
end; { SetAutoCalcFields }

function TfrxAsaDataSet.GetCursorType: TAsaCursorType;
begin
 Result := FQuery.CursorType;
end; { GetCursorType }

procedure TfrxAsaDataSet.SetCursorType(const Value: TAsaCursorType);
begin
 FQuery.CursorType := Value;
end; { SetCursorType }

function TfrxAsaDataSet.GetSyncInserts: Boolean;
begin
 Result := FQuery.SyncInserts;
end; { GetSyncInserts }

procedure TfrxAsaDataSet.SetSyncInserts(const Value: Boolean);
begin
 FQuery.SyncInserts := Value;
end; { SetSyncInserts }

function TfrxAsaDataSet.GetStillInProc: Boolean;
begin
 Result := FQuery.StillInProc;
end; { GetStillInProc }

procedure TfrxAsaDataSet.ExecSQL;
begin
 FQuery.ExecSQL;
end; { ExecSQL }

function TfrxAsaDataSet.LocateNext(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
 Result := FQuery.LocateNext(KeyFields, KeyValues, Options);
end; { LocateNext }

procedure TfrxAsaDataSet.Resume;
begin
 FQuery.Resume;
end; { Resume }

{$IFDEF QBUILDER}
function TfrxAsaDataSet.QBEngine: TfqbEngine;
begin
 Result := TfrxEngineASA.Create(nil);
 TfrxEngineASA(Result).Query.Session := FQuery.Session;
end; { QBEngine }
{$ENDIF}

{$IFDEF QBUILDER}
constructor TfrxEngineASA.Create(AOwner: TComponent);
begin
 inherited;
 FQuery := TAsaDataSet.Create(Self);
end; { Create }

destructor TfrxEngineASA.Destroy;
begin
 FreeAndNil(FQuery);
 inherited
end; { Destroy }

procedure TfrxEngineASA.ReadFieldList(const ATableName :String; var AFieldList :TfqbFieldList);
var
  TempTable :TAsaDataSet;
  Fields    :TFieldDefs;
  i         :Integer;
  tmpField  :TfqbField;

begin

 if not Assigned(FQuery.Session) then Exit;

 AFieldList.Clear;
 TempTable := TAsaDataSet.Create(Self);
 try

   TempTable.Session := FQuery.Session;
   TempTable.SQL.Text := 'select * from '+ATableName;
   Fields := TempTable.FieldDefs;

   try

     TempTable.Active := True;
     tmpField:= TfqbField(AFieldList.Add);
     tmpField.FieldName := '*';

     for i := 0 to Fields.Count - 1 do begin
       tmpField := TfqbField(AFieldList.Add);
       tmpField.FieldName := Fields.Items[i].Name;
       tmpField.FieldType := Ord(Fields.Items[i].DataType)
     end; { for }

   except
   end; { try ... except }

 finally
   FreeAndNil(TempTable);
 end; { try .. finally }

end; { ReadFieldList }

procedure TfrxEngineASA.ReadTableList(ATableList :TStrings);
begin

 ATableList.BeginUpdate;
 ATableList.Clear;
 try
   if Assigned(FQuery.Session) then
     NdbGetTableNames(FQuery.Session.LoginDatabase, ShowSystemTables, True, ATableList, TAsaDataset);
 finally
   ATableList.EndUpdate;
 end; { try }

end; { ReadTableList }

function TfrxEngineASA.ResultDataSet: TDataSet;
begin
 Result := FQuery;
end; { ResultDataSet }

procedure TfrxEngineASA.SetSQL(const Value: string);
begin
 FQuery.SQL.Text := Value;
end; { SetSQL }
{$ENDIF}

{ frxParamsToTParams }
procedure frxParamsToTParams(Query: TfrxAsaDataSet; Params: TNativeParams);

 function ConvPT(oFieldType :TFieldType) :TNativeParamDataType;
 begin

   case oFieldType of
     ftFloat    :Result := ntFloat;
     ftInteger  :Result := ntInteger;
     ftSmallInt :Result := ntInteger;
     ftCurrency :Result := ntCurrency;
     ftDate     :Result := ntDate;
     ftDateTime :Result := ntDateTime;
     ftTime     :Result := ntTime;
     ftBoolean  :Result := ntBoolean;
     ftString   :Result := ntString;
     ftBlob     :Result := ntBlob;
     ftMemo     :Result := ntMemo;
   else
     Result := ntUnknown
   end; { case }

 end; { ConvPT }

var
  i: Integer;
  Item: TfrxParamItem;

begin

 for i := 0 to Params.Count - 1 do
   if Query.Params.IndexOf(Params[i].Name) <> -1 then begin
     Item := Query.Params[Query.Params.IndexOf(Params[i].Name)];
     Params[i].Clear;
     Params[i].DataType := ConvPT(Item.DataType);
     { Bound should be True in design mode }
     if not (Query.IsLoading or Query.IsDesigning) then
       Params[i].Bound := False
     else begin
       if Trim(Item.Expression) <> '' then
         Params[i].Value := 0;
       Params[i].Bound := True;
     end; { if }
     if Trim(Item.Expression) <> '' then
       if not (Query.IsLoading or Query.IsDesigning) then
         if Query.Report <> nil then begin
           Query.Report.CurObject := Query.Name;
           Item.Value := Query.Report.Calc(Item.Expression);
         end; { if }
     if not VarIsEmpty(Item.Value) then begin
       Params[i].Bound := True;
       if Params[i].DataType in [ntDate, ntTime, ntDateTime] then
         Params[i].Value := Item.Value
       else
         Params[i].Text := VarToStr(Item.Value);
     end; { if }
   end; { if }

end; { frxParamsToTParams }

var
  Bmp1 :TBitmap;
  Bmp2 :TBitmap;
  Bmp3 :TBitmap;

initialization
  Bmp1 := TBitmap.Create;
  Bmp1.LoadFromResourceName(hInstance, 'FRXASACOMPONENTS');
  frxObjects.RegisterCategory('NDB', Bmp1, 'NativeDB');

  Bmp2 := TBitmap.Create;
  Bmp2.LoadFromResourceName(hInstance, 'FRX_ASASESSION');
  frxObjects.RegisterObject1(TfrxAsaSession, Bmp2, 'NativeDB: frxAsaSession', 'NDB');

  Bmp3 := TBitmap.Create;
  Bmp3.LoadFromResourceName(hInstance, 'FRX_ASADATASET');
  frxObjects.RegisterObject1(TfrxAsaDataSet, Bmp3, 'NativeDB: frxAsaDataSet', 'NDB');

finalization
  frxObjects.UnRegister(TfrxAsaSession);
  frxObjects.UnRegister(TfrxAsaDataSet);
  FreeAndNil(Bmp1);
  FreeAndNil(Bmp2);
  FreeAndNil(Bmp3);

end.