{**********************************************}
{                                              }
{             FastReport v5.0                  }
{          NativeDB components RTTI            }
{                                              }
{         Copyright (c) 1998-2014              }
{           by Fast Reports Inc.               }
{                                              }
{ Copyright (c) 2000-2014 by Stalker SoftWare  }
{                                              }
{**********************************************}

unit frxASARTTI;

interface

uses
  Windows, Classes, SysUtils, Forms, fs_iInterpreter, fs_iDBRTTI, DB
{$IFDEF Delphi6}
, Variants
{$ENDIF};

{$I frx.inc}

implementation

uses
  frxASAComponents;

type
  TFunctions = class(TfsRTTIModule)
  private
    function CallMethod(Instance: TObject; ClassType: TClass; const MethodName: String; Caller: TfsMethodHelper): Variant;
    function GetProp(Instance: TObject; ClassType: TClass; const PropName: String): Variant;
  public
    constructor Create(AScript: TfsScript); override;
  end;

{ TFunctions }

constructor TFunctions.Create(AScript: TfsScript);
begin

 inherited Create(AScript);

 with AScript do begin

    with AddClass(TfrxAsaSession, 'TfrxCustomDatabase') do
      AddProperty('AsaSession', 'TAsaSession', GetProp, nil);

    with AddClass(TfrxAsaDataSet, 'TfrxCustomQuery') do begin
      AddProperty('Query', 'TSDQuery', GetProp, nil);
      AddProperty('StillInProc', 'Boolean', GetProp, nil);
      AddMethod('function ExecSQL :Boolean', CallMethod);
      AddMethod('function LocateNext(const KeyFields: string; const KeyValues: Variant;Options: TLocateOptions): Boolean', CallMethod);
      AddMethod('procedure Resume', CallMethod);
    end; { with }

    AddEnum('TAsaServerType','stClient,stServer,stWorkgroup');
    AddEnum('TAsaTransIsolation','atReadCommitted,atReadUncommitted,atReapeatableRead,atUnchanged,atSerializable');
    AddEnum('TAsaCursorType', 'actDefault,actDynamicScroll,actFixedScroll,actInsensitive,actNoScroll,actUnique');

 end; { with }

end; { Create }

function TFunctions.CallMethod(Instance: TObject; ClassType: TClass; const MethodName: String; Caller: TfsMethodHelper): Variant;

  function IntToLocateOptions(i: Integer): TLocateOptions;
  begin
   Result := [];
   if (i and 1) <> 0 then
     Result := Result + [loCaseInsensitive];
   if (i and 2) <> 0 then
     Result := Result + [loPartialKey];
  end; { IntToLocateOptions }

begin

 Result := 0;

 if ClassType = TfrxAsaDataSet then begin

   if MethodName = 'EXECSQL' then
     TfrxAsaDataSet(Instance).ExecSQL
  else
  if MethodName = 'LOCATENEXT' then
    Result := TfrxAsaDataSet(Instance).LocateNext(Caller.Params[0], Caller.Params[1], IntToLocateOptions(Caller.Params[2]))
  else
  if MethodName = 'RESUME' then
    TfrxAsaDataSet(Instance).Resume
    
 end

end; { CallMethod }

function TFunctions.GetProp(Instance: TObject; ClassType: TClass; const PropName: String): Variant;
begin

 Result := 0;

 if ClassType = TfrxAsaSession then begin
   if PropName = 'ASASESSION' then
     Result := frxInteger(TfrxAsaSession(Instance).AsaSession)
 end
 else
 if ClassType = TfrxAsaDataSet then begin
   if PropName = 'QUERY' then
     Result := frxInteger(TfrxAsaDataSet(Instance).Query)
   else
   if PropName = 'STILLINPROC' then
     Result := TfrxAsaDataSet(Instance).StillInProc
 end

end; { GetProp }

initialization
  fsRTTIModules.Add(TFunctions);

finalization
  if fsRTTIModules <> nil then
    fsRTTIModules.Remove(TFunctions);

end.
